==> 0.0.54.tar.gz: lib/remote/base_service.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:hive/hive.dart';
import 'package:sharing/remote/model/data_extension_model.dart';
import 'constants.dart';
import 'logger.dart';
import 'model/base_list_response.dart';
import 'model/base_object_response.dart';
import 'network/fetch_data.dart';
import 'network/either.dart';
import 'network/fetch_failure.dart';

abstract class BaseService {
  final logger = getLogger('NetWorkTask');

  @protected
  Future<Either<F, T>> safeCall<F, T>({
    required Future<T> Function() call,
    required F Function() onNetworkError,
    required F Function(Response? response)
        onResponseError, // ignore: always_specify_types
    required F Function(Exception e) onUnknownError,
  }) async {
    try {
      final T result = await call();
      return right(result);
    } on DioError catch (e) {
      // logger.e("BaseService.safeCall: ", e);
      switch (e.type) {
        case DioErrorType.connectTimeout:
          return left(onNetworkError());
        case DioErrorType.sendTimeout:
          return left(onNetworkError());
        case DioErrorType.receiveTimeout:
          return left(onNetworkError());
        case DioErrorType.response:
          return left(onResponseError(e.response));
        case DioErrorType.cancel:
          return left(onNetworkError());
        case DioErrorType.other:
          if (e.error is SocketException) {
            return left(onNetworkError());
          }
          return left(onUnknownError(e));
      }
    }
  }

  @protected
  Future<Either<FetchFailure, T>> safeFetch<T>(
      Future<T> Function() call) async {
    return safeCall(
      call: call,
      onNetworkError: () => const FetchFailure.networkError(),
      onResponseError: (Response? response) {
        try {
          if (response != null && response.statusCode != null) {
            int? statusCode = response.statusCode ?? 00;
            var jsonString = json.encode(response.data["errors"]);
            var msg = List<String>.from(json.decode(jsonString));
            var data = response.data["code_name"];
            if (statusCode == 401) {
              return FetchFailure.login(msg.toList(), data);
            } else if (statusCode >= 400 && statusCode < 500) {
              try {
                return FetchFailure.warningError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            } else if (statusCode >= 500 && statusCode < 600) {
              try {
                return FetchFailure.serverError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            }
          }
          return FetchFailure.unknownError(101, "Unknown", "Unknown");
        } catch (e) {
          var status = response?.statusCode ?? 101;
          String msg = response?.statusMessage ?? "";
          var data = response?.statusMessage ?? "";
          return FetchFailure.serverError(status, [msg], data);
        }
      },
      onUnknownError: (Exception e) =>
          FetchFailure.unknownError(101, "Unknown", "Unknown"),
    );
  }

  @protected
  Future<Either<FetchFailure, T>> safeFetchV2<T>(
      Future<T> Function() call) async {
    return safeCall(
      call: call,
      onNetworkError: () => const FetchFailure.networkError(),
      onResponseError: (Response? response) {
        try {
          if (response != null && response.statusCode != null) {
            int? statusCode = response.statusCode ?? 00;
            var msg = response.data["msg"];
            var data = response.data["data"];
            if (statusCode == 401) {
              return FetchFailure.login([data], data);
            } else if (statusCode >= 400 && statusCode < 500) {
              try {
                return FetchFailure.warningError(statusCode, [msg], data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            } else if (statusCode >= 500 && statusCode < 600) {
              try {
                return FetchFailure.serverError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            }
          }
          return FetchFailure.unknownError(101, "Unknown", "Unknown");
        } catch (e) {
          var status = response?.statusCode ?? 101;
          String msg = response?.statusMessage ?? "";
          var data = response?.statusMessage ?? "";
          return FetchFailure.serverError(status, [msg], data);
        }
      },
      onUnknownError: (Exception e) =>
          FetchFailure.unknownError(101, "Unknown", "Unknown"),
    );
  }

  //=================== SAFE CALL FORM DATA LIST =====================//

  Future<void> safeCallListStatus<T>(
      {required Either<FetchFailure, BaseListResponse<T>?> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<List<T>>) funDataServer,
      Function(FetchData<List<T>>)? funSuccess,
      bool isLoadCache = true,
      String keyCache = "",
      Function(List<T>?)? funDataCache,
      bool isShowError = true}) async {
    try {
      if (funDataCache != null && keyCache.isNotEmpty == true && isLoadCache) {
        await safeResponseDataCache<List<T>>(
            keyCache: keyCache,
            funCallBack: (data) {
              isShowError = false;
              funDataCache(data);
            });
      }
      if (response.get != null) {
        await safeResponseDataServer<List<T>>(
            dataServer: response.get?.data,
            funCallBack: (data) async {
              if (data?.isNotEmpty == true) {
                if (funSuccess != null) {
                  funSuccess(FetchData<List<T>>(data: data));
                }
                return funDataServer(FetchData<List<T>>(data: data));
              } else {
                return funDataServer(FetchData<List<T>>(data: []));
              }
            });
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  Future<void> safeCallReportStatus<T>(
      {required Either<FetchFailure, BaseListResponse<T>?> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<List<T>>) funDataServer,
      Function(FetchData<List<T>>)? funSuccess,
      bool isShowError = true}) async {
    try {
      if (response.get != null) {
        await safeResponseDataServer<List<T>>(
            dataServer: response.get?.data,
            funCallBack: (data) async {
              if (data?.isNotEmpty == true) {
                final dataEx = response.get?.data_extension;
                if (funSuccess != null) {
                  funSuccess(FetchData<List<T>>(data: data, dataEx: dataEx));
                }
                return funDataServer(
                    FetchData<List<T>>(data: data, dataEx: dataEx));
              } else {
                return funError(await safeResponseError(
                    typeError: TypeError.ERROR_NO_DATA,
                    isShowError: isShowError));
              }
            });
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  //=================== SAFE CALL FORM DATA OBJECT =====================//

  Future<void> safeCallObjectStatus<T>(
      {required Either<FetchFailure, BaseObjectResponse<T?>> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<T>) funDataServer,
      Function(FetchData<T>)? funSuccess,
      String keyCache = "",
      Function(T?)? funDataCache,
      bool isShowError = true}) async {
    try {
      if (funDataCache != null && keyCache.isNotEmpty == true) {
        await safeResponseDataCache<T>(
            keyCache: keyCache,
            funCallBack: (data) {
              isShowError = false;
              funDataCache(data);
            });
      }
      if (response.get != null) {
        safeResponseDataServer<T>(
            dataServer: response.get?.data,
            funCallBack: (data) async {
              if (data != null) {
                safeDataCache(
                    isDataCache: keyCache.isNotEmpty == true,
                    funCallBack: () {
                      saveCacheData(key: keyCache, data: data);
                    });
                final msg = response.get?.message;
                if (funSuccess != null) {
                  funSuccess(FetchData<T>(data: data, msg: msg));
                }
                return funDataServer(FetchData<T>(data: data, msg: msg));
              } else {
                return funError(await safeResponseError(
                    typeError: TypeError.ERROR_NO_DATA,
                    isShowError: isShowError));
              }
            });
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

//=================== SAFE CALL FORM DATA MESSAGE =====================//

  Future<Either<FetchFailure, String?>> safeCallMessageStatus<T>(
      {required Either<FetchFailure, BaseObjectResponse<T>> response,
      bool isMessageFromData = true,
      bool isShowError = true}) async {
    try {
      if (response.get != null) {
        if (response.get?.success == true) {
          if (isMessageFromData) {
            return response.map((data) => data.data.toString());
          } else {
            return response.map((data) => data.message.toString());
          }
        } else {
          final msg = response.get?.data ?? "ERROR";
          return left(await safeResponseError(
              typeError: TypeError.ERROR_SERVER,
              errorServer: FetchFailure.warningError(200, [msg.toString()], ""),
              isShowError: isShowError));
        }
      } else {
        return left(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      print(e.toString());
      final failure = returnFailure(e);
      return left(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  //=================== SAFE CALL FORM DATA CACHE =====================//

  Future<Either<String, T>> safeCallCacheStatus<T>(T? response) async {
    try {
      if (response != null) {
        return await right(response);
      } else {
        return await left("Error");
      }
    } catch (e) {
      final failure = returnFailure(e);
      return left(failure);
    }
  }

  Future<Either<FetchFailure, T>> safeCallLocalCache<T>(T? response) async {
    try {
      if (response != null) {
        return await right(response);
      } else {
        return await left(const FetchFailure.nothing());
      }
    } catch (e) {
      return await left(const FetchFailure.nothing());
    }
  }

//=================== SAFE CALL HIVE BOX CACHE =====================//

  Future<void> saveCacheData<T>({required String? key, T? data}) async {
    if (key?.isNotEmpty == true) {
      final cacheBox = await Hive.openBox(key!);
      cacheBox.put(key, data);
    }
  }

  Future<void> safeResponseDataCache<T>(
      {required String? keyCache, required Function(T?) funCallBack}) async {
    if (keyCache?.isNotEmpty == true) {
      final cacheData = await responseCacheData<T>(key: keyCache!);
      if (cacheData != null) {
        funCallBack(cacheData);
      }
    }
  }

//=================== SAFE RESPONSE LEFT OR CRASH =====================//

  Future<T?> responseCacheData<T>({required String key}) async {
    T? dataCache;
    final cacheBox = await Hive.openBox(key);
    try {
      if (cacheBox.isNotEmpty == true) {
        if (cacheBox.values.single != null) {
          dataCache = cacheBox.get(key);
          return Future.value(dataCache);
        }
      }
    } catch (e) {
      cacheBox.clear();
      return Future.value(dataCache);
    }
    return Future.value(dataCache);
  }

  Future<void> safeResponseDataServer<T>({
    required T? dataServer,
    required Function(T?) funCallBack,
  }) async {
    funCallBack(dataServer);
  }

  void safeDataCache<T>(
      {required Function() funCallBack, bool? isDataCache = false}) {
    if (isDataCache == true) {
      funCallBack();
    }
  }
//=================== SAFE RESPONSE ERROR =====================//

  Future<FetchFailure> safeResponseError(
      {required TypeError typeError,
      FetchFailure? errorServer,
      FetchFailure? errorUnknow,
      bool? isShowError = true}) async {
    switch (typeError) {
      case TypeError.ERROR_NO_DATA:
        if (isShowError == true) {
          return await FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      case TypeError.ERROR_SERVER:
        if (isShowError == true) {
          return await errorServer ?? FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      case TypeError.ERROR_UNKNOWN:
        if (isShowError == true) {
          return await errorUnknow ?? FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      default:
        if (isShowError == true) {
          return await const FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
    }
  }
}

enum TypeError { ERROR_NO_DATA, ERROR_SERVER, ERROR_UNKNOWN }

==> 0.0.63.tar.gz: lib/remote/base_service.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:hive/hive.dart';
import 'constants.dart';
import 'logger.dart';
import 'model/base_list_response.dart';
import 'model/base_object_response.dart';
import 'network/fetch_data.dart';
import 'network/either.dart';
import 'network/fetch_failure.dart';

abstract class BaseService {
  final logger = getLogger('NetWorkTask');

  @protected
  Future<Either<F, T>> safeCall<F, T>({
    required Future<T> Function() call,
    required F Function() onNetworkError,
    required F Function(Response? response)
        onResponseError, // ignore: always_specify_types
    required F Function(Exception e) onUnknownError,
  }) async {
    try {
      final T result = await call();
      return right(result);
    } on DioError catch (e) {
      // logger.e("BaseService.safeCall: ", e);
      switch (e.type) {
        case DioErrorType.connectTimeout:
          return left(onNetworkError());
        case DioErrorType.sendTimeout:
          return left(onNetworkError());
        case DioErrorType.receiveTimeout:
          return left(onNetworkError());
        case DioErrorType.response:
          return left(onResponseError(e.response));
        case DioErrorType.cancel:
          return left(onNetworkError());
        case DioErrorType.other:
          if (e.error is SocketException) {
            return left(onNetworkError());
          }
          return left(onUnknownError(e));
      }
    }
  }

  @protected
  Future<Either<FetchFailure, T>> safeFetch<T>(
      Future<T> Function() call) async {
    return safeCall(
      call: call,
      onNetworkError: () => const FetchFailure.networkError(),
      onResponseError: (Response? response) {
        try {
          if (response != null && response.statusCode != null) {
            int? statusCode = response.statusCode ?? 00;
            var jsonString = json.encode(response.data["errors"]);
            var msg = List<String>.from(json.decode(jsonString));
            var data = response.data["code_name"];
            if (statusCode == 401) {
              return FetchFailure.login(msg.toList(), data);
            } else if (statusCode >= 400 && statusCode < 500) {
              try {
                return FetchFailure.warningError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            } else if (statusCode >= 500 && statusCode < 600) {
              try {
                return FetchFailure.serverError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            }
          }
          return FetchFailure.unknownError(101, "Unknown", "Unknown");
        } catch (e) {
          var status = response?.statusCode ?? 101;
          String msg = response?.statusMessage ?? "";
          var data = response?.statusMessage ?? "";
          return FetchFailure.serverError(status, [msg], data);
        }
      },
      onUnknownError: (Exception e) =>
          FetchFailure.unknownError(101, "Unknown", "Unknown"),
    );
  }

  @protected
  Future<Either<FetchFailure, T>> safeFetchV2<T>(
      Future<T> Function() call) async {
    return safeCall(
      call: call,
      onNetworkError: () => const FetchFailure.networkError(),
      onResponseError: (Response? response) {
        try {
          if (response != null && response.statusCode != null) {
            int? statusCode = response.statusCode ?? 00;
            var msg = response.data["msg"];
            var data = response.data["data"];
            if (statusCode == 401) {
              return FetchFailure.login([data], data);
            } else if (statusCode >= 400 && statusCode < 500) {
              try {
                return FetchFailure.warningError(statusCode, [msg], data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            } else if (statusCode >= 500 && statusCode < 600) {
              try {
                return FetchFailure.serverError(statusCode, msg, data);
              } catch (e) {
                msg = [response.statusMessage.toString()];
                data = response.data.toString();
                return FetchFailure.warningError(statusCode, msg, data);
              }
            }
          }
          return FetchFailure.unknownError(101, "Unknown", "Unknown");
        } catch (e) {
          var status = response?.statusCode ?? 101;
          String msg = response?.statusMessage ?? "";
          var data = response?.statusMessage ?? "";
          return FetchFailure.serverError(status, [msg], data);
        }
      },
      onUnknownError: (Exception e) =>
          FetchFailure.unknownError(101, "Unknown", "Unknown"),
    );
  }

  //=================== SAFE CALL FORM DATA LIST =====================//

  Future<void> safeCallListStatus<T>(
      {required Either<FetchFailure, BaseListResponse<T>?> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<List<T>>) funDataServer,
      Function(FetchData<List<T>>)? funSuccess,
      bool isLoadCache = true,
      String keyCache = "",
      Function(List<T>?)? funDataCache,
      bool isShowError = true}) async {
    try {
      if (funDataCache != null && keyCache.isNotEmpty == true && isLoadCache) {
        await safeResponseDataCache<List<T>>(
            keyCache: keyCache,
            funCallBack: (data) {
              isShowError = false;
              funDataCache(data);
            });
      }
      if (response.get != null) {
        if (response.get?.success == true) {
          await safeResponseDataServer<List<T>>(
              dataServer: response.get?.data,
              funCallBack: (data) async {
                if (data?.isNotEmpty == true) {
                  safeDataCache(
                      isDataCache: keyCache.isNotEmpty == true && isLoadCache,
                      funCallBack: () {
                        saveCacheData(key: keyCache, data: data);
                      });
                  final msg = response.get?.message;
                  if (funSuccess != null) {
                    funSuccess(FetchData<List<T>>(data: data, msg: msg));
                  }
                  return funDataServer(
                      FetchData<List<T>>(data: data, msg: msg));
                } else {
                  return funDataServer(FetchData<List<T>>(data: []));
                }
              });
        } else {
          final msg = response.get?.message.toString() ?? "Error";
          final data = response.get?.data.toString() ?? "Error";
          return funError(await safeResponseError(
              typeError: TypeError.ERROR_WARNING,
              errorServer: FetchFailure.warningError(200, [msg], data),
              isShowError: isShowError));
        }
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  Future<void> safeCallReportStatus<T>(
      {required Either<FetchFailure, BaseListResponse<T>?> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<List<T>>) funDataServer,
      Function(FetchData<List<T>>)? funSuccess,
      bool isShowError = true}) async {
    try {
      if (response.get != null) {
        await safeResponseDataServer<List<T>>(
            dataServer: response.get?.data,
            funCallBack: (data) async {
              if (data?.isNotEmpty == true) {
                final dataEx = response.get?.data_extension;
                if (funSuccess != null) {
                  funSuccess(FetchData<List<T>>(data: data, dataEx: dataEx));
                }
                return funDataServer(
                    FetchData<List<T>>(data: data, dataEx: dataEx));
              } else {
                return funError(await safeResponseError(
                    typeError: TypeError.ERROR_NO_DATA,
                    isShowError: isShowError));
              }
            });
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  //=================== SAFE CALL FORM DATA OBJECT =====================//

  Future<void> safeCallObjectStatus<T>(
      {required Either<FetchFailure, BaseObjectResponse<T?>> response,
      required Function(FetchFailure) funError,
      required Function(FetchData<T>) funDataServer,
      Function(FetchData<T>)? funSuccess,
      String keyCache = "",
      Function(T?)? funDataCache,
      bool isShowError = true}) async {
    try {
      if (funDataCache != null && keyCache.isNotEmpty == true) {
        await safeResponseDataCache<T>(
            keyCache: keyCache,
            funCallBack: (data) {
              isShowError = false;
              funDataCache(data);
            });
      }
      if (response.get != null) {
        if (response.get?.success == true) {
          safeResponseDataServer<T>(
              dataServer: response.get?.data,
              funCallBack: (data) async {
                if (data != null) {
                  safeDataCache(
                      isDataCache: keyCache.isNotEmpty == true,
                      funCallBack: () {
                        saveCacheData(key: keyCache, data: data);
                      });
                  final msg = response.get?.message;
                  if (funSuccess != null) {
                    funSuccess(FetchData<T>(data: data, msg: msg));
                  }
                  return funDataServer(FetchData<T>(data: data, msg: msg));
                } else {
                  return funError(await safeResponseError(
                      typeError: TypeError.ERROR_NO_DATA,
                      isShowError: isShowError));
                }
              });
        } else {
          final msg = response.get?.message.toString() ?? "Error";
          final data = response.get?.data.toString() ?? "Error";
          return funError(await safeResponseError(
              typeError: TypeError.ERROR_WARNING,
              errorServer: FetchFailure.warningError(200, [msg], data),
              isShowError: isShowError));
        }
      } else {
        return funError(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      final failure = returnFailure(e);
      return funError(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

//=================== SAFE CALL FORM DATA MESSAGE =====================//

  Future<Either<FetchFailure, String?>> safeCallMessageStatus<T>(
      {required Either<FetchFailure, BaseObjectResponse<T>> response,
      bool isMessageFromData = true,
      bool isShowError = true}) async {
    try {
      if (response.get != null) {
        if (response.get?.success == true) {
          if (isMessageFromData) {
            return response.map((data) => data.data.toString());
          } else {
            return response.map((data) => data.message.toString());
          }
        } else {
          final msg = response.get?.message.toString() ?? "Error";
          final data = response.get?.data.toString() ?? "Error";
          return left(await safeResponseError(
              typeError: TypeError.ERROR_WARNING,
              errorServer: FetchFailure.warningError(200, [msg], data),
              isShowError: isShowError));
        }
      } else {
        return left(await safeResponseError(
            typeError: TypeError.ERROR_SERVER,
            errorServer: response.leftOrCrash,
            isShowError: isShowError));
      }
    } catch (e) {
      print(e.toString());
      final failure = returnFailure(e);
      return left(await safeResponseError(
          typeError: TypeError.ERROR_UNKNOWN,
          errorServer: FetchFailure.crashError(100, failure, "Error"),
          isShowError: isShowError));
    }
  }

  //=================== SAFE CALL FORM DATA CACHE =====================//

  Future<Either<String, T>> safeCallCacheStatus<T>(T? response) async {
    try {
      if (response != null) {
        return await right(response);
      } else {
        return await left("Error");
      }
    } catch (e) {
      final failure = returnFailure(e);
      return left(failure);
    }
  }

  Future<Either<FetchFailure, T>> safeCallLocalCache<T>(T? response) async {
    try {
      if (response != null) {
        return await right(response);
      } else {
        return await left(const FetchFailure.nothing());
      }
    } catch (e) {
      return await left(const FetchFailure.nothing());
    }
  }

//=================== SAFE CALL HIVE BOX CACHE =====================//

  Future<void> saveCacheData<T>({required String? key, T? data}) async {
    if (key?.isNotEmpty == true) {
      final cacheBox = await Hive.openBox(key!);
      cacheBox.put(key, data);
    }
  }

  Future<void> safeResponseDataCache<T>(
      {required String? keyCache, required Function(T?) funCallBack}) async {
    if (keyCache?.isNotEmpty == true) {
      final cacheData = await responseCacheData<T>(key: keyCache!);
      if (cacheData != null) {
        funCallBack(cacheData);
      }
    }
  }

//=================== SAFE RESPONSE LEFT OR CRASH =====================//

  Future<T?> responseCacheData<T>({required String key}) async {
    T? dataCache;
    final cacheBox = await Hive.openBox(key);
    try {
      if (cacheBox.isNotEmpty == true) {
        if (cacheBox.values.single != null) {
          dataCache = cacheBox.get(key);
          return Future.value(dataCache);
        }
      }
    } catch (e) {
      cacheBox.clear();
      return Future.value(dataCache);
    }
    return Future.value(dataCache);
  }

  Future<void> safeResponseDataServer<T>({
    required T? dataServer,
    required Function(T?) funCallBack,
  }) async {
    funCallBack(dataServer);
  }

  void safeDataCache<T>(
      {required Function() funCallBack, bool? isDataCache = false}) {
    if (isDataCache == true) {
      funCallBack();
    }
  }
//=================== SAFE RESPONSE ERROR =====================//

  Future<FetchFailure> safeResponseError(
      {required TypeError typeError,
      FetchFailure? errorServer,
      FetchFailure? errorWarning,
      FetchFailure? errorUnknow,
      bool? isShowError = true}) async {
    switch (typeError) {
      case TypeError.ERROR_NO_DATA:
        if (isShowError == true) {
          return await FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      case TypeError.ERROR_SERVER:
        if (isShowError == true) {
          return await errorServer ?? FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      case TypeError.ERROR_WARNING:
        if (isShowError == true) {
          return await errorServer ?? FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      case TypeError.ERROR_UNKNOWN:
        if (isShowError == true) {
          return await errorUnknow ?? FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
      default:
        if (isShowError == true) {
          return await const FetchFailure.noData();
        } else {
          return await const FetchFailure.nothing();
        }
    }
  }
}

enum TypeError { ERROR_NO_DATA, ERROR_WARNING, ERROR_SERVER, ERROR_UNKNOWN }
